@@ -1,7 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 Kurt Pattyn <pattyn.kurt@gmail.com>.
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtWebSockets module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -84,7 +84,7 @@ QWebSocketPrivate::QWebSocketPrivate(const QString &origin, QWebSocketProtocol::
                                      QWebSocket *pWebSocket) :
     QObjectPrivate(),
     q_ptr(pWebSocket),
-    m_pSocket(),
+    m_pSocket(Q_NULLPTR),
     m_errorString(),
     m_version(version),
     m_resourceName(),
@@ -105,7 +105,8 @@ QWebSocketPrivate::QWebSocketPrivate(const QString &origin, QWebSocketProtocol::
     m_dataProcessor(),
     m_configuration(),
     m_pMaskGenerator(&m_defaultMaskGenerator),
-    m_defaultMaskGenerator()
+    m_defaultMaskGenerator(),
+    m_handshakeState(NothingDoneState)
 {
 }
 
@@ -137,7 +138,8 @@ QWebSocketPrivate::QWebSocketPrivate(QTcpSocket *pTcpSocket, QWebSocketProtocol:
     m_dataProcessor(),
     m_configuration(),
     m_pMaskGenerator(&m_defaultMaskGenerator),
-    m_defaultMaskGenerator()
+    m_defaultMaskGenerator(),
+    m_handshakeState(NothingDoneState)
 {
 }
 
@@ -152,7 +154,7 @@ void QWebSocketPrivate::init()
     m_pMaskGenerator->seed();
 
     if (m_pSocket) {
-        makeConnections(m_pSocket.data());
+        makeConnections(m_pSocket);
     }
 }
 
@@ -160,12 +162,19 @@ void QWebSocketPrivate::init()
     \internal
 */
 QWebSocketPrivate::~QWebSocketPrivate()
+{
+}
+
+/*!
+    \internal
+*/
+void QWebSocketPrivate::closeGoingAway()
 {
     if (!m_pSocket)
         return;
     if (state() == QAbstractSocket::ConnectedState)
         close(QWebSocketProtocol::CloseCodeGoingAway, QWebSocket::tr("Connection closed"));
-    releaseConnections(m_pSocket.data());
+    releaseConnections(m_pSocket);
 }
 
 /*!
@@ -260,7 +269,7 @@ void QWebSocketPrivate::ignoreSslErrors()
 {
     m_configuration.m_ignoreSslErrors = true;
     if (Q_LIKELY(m_pSocket)) {
-        QSslSocket *pSslSocket = qobject_cast<QSslSocket *>(m_pSocket.data());
+        QSslSocket *pSslSocket = qobject_cast<QSslSocket *>(m_pSocket);
         if (Q_LIKELY(pSslSocket))
             pSslSocket->ignoreSslErrors();
     }
@@ -332,17 +341,17 @@ void QWebSocketPrivate::open(const QUrl &url, bool mask)
 {
     //just delete the old socket for the moment;
     //later, we can add more 'intelligent' handling by looking at the URL
-    //m_pSocket.reset();
+
     Q_Q(QWebSocket);
     if (!url.isValid() || url.toString().contains(QStringLiteral("\r\n"))) {
         setErrorString(QWebSocket::tr("Invalid URL."));
         Q_EMIT q->error(QAbstractSocket::ConnectionRefusedError);
         return;
     }
-    QTcpSocket *pTcpSocket = m_pSocket.take();
-    if (pTcpSocket) {
-        releaseConnections(pTcpSocket);
-        pTcpSocket->deleteLater();
+    if (m_pSocket) {
+        releaseConnections(m_pSocket);
+        m_pSocket->deleteLater();
+        m_pSocket = Q_NULLPTR;
     }
     //if (m_url != url)
     if (Q_LIKELY(!m_pSocket)) {
@@ -351,8 +360,9 @@ void QWebSocketPrivate::open(const QUrl &url, bool mask)
         m_isClosingHandshakeSent = false;
 
         setRequestUrl(url);
-        QString resourceName = url.path();
-        if (resourceName.contains(QStringLiteral("\r\n"))) {
+        QString resourceName = url.path(QUrl::FullyEncoded);
+        // Check for encoded \r\n
+        if (resourceName.contains(QStringLiteral("%0D%0A"))) {
             setRequestUrl(QUrl());  //clear requestUrl
             setErrorString(QWebSocket::tr("Invalid resource name."));
             Q_EMIT q->error(QAbstractSocket::ConnectionRefusedError);
@@ -362,7 +372,7 @@ void QWebSocketPrivate::open(const QUrl &url, bool mask)
             if (!resourceName.endsWith(QChar::fromLatin1('?'))) {
                 resourceName.append(QChar::fromLatin1('?'));
             }
-            resourceName.append(url.query());
+            resourceName.append(url.query(QUrl::FullyEncoded));
         }
         if (resourceName.isEmpty())
             resourceName = QStringLiteral("/");
@@ -377,15 +387,15 @@ void QWebSocketPrivate::open(const QUrl &url, bool mask)
                 setErrorString(message);
                 Q_EMIT q->error(QAbstractSocket::UnsupportedSocketOperationError);
             } else {
-                QSslSocket *sslSocket = new QSslSocket;
-                m_pSocket.reset(sslSocket);
+                QSslSocket *sslSocket = new QSslSocket(q_ptr);
+                m_pSocket = sslSocket;
                 if (Q_LIKELY(m_pSocket)) {
                     m_pSocket->setSocketOption(QAbstractSocket::LowDelayOption, 1);
                     m_pSocket->setSocketOption(QAbstractSocket::KeepAliveOption, 1);
                     m_pSocket->setReadBufferSize(m_readBufferSize);
                     m_pSocket->setPauseMode(m_pauseMode);
 
-                    makeConnections(m_pSocket.data());
+                    makeConnections(m_pSocket);
                     setSocketState(QAbstractSocket::ConnectingState);
 
                     sslSocket->setSslConfiguration(m_configuration.m_sslConfiguration);
@@ -406,14 +416,14 @@ void QWebSocketPrivate::open(const QUrl &url, bool mask)
         } else
     #endif
         if (url.scheme() == QStringLiteral("ws")) {
-            m_pSocket.reset(new QTcpSocket);
+            m_pSocket = new QTcpSocket(q_ptr);
             if (Q_LIKELY(m_pSocket)) {
                 m_pSocket->setSocketOption(QAbstractSocket::LowDelayOption, 1);
                 m_pSocket->setSocketOption(QAbstractSocket::KeepAliveOption, 1);
                 m_pSocket->setReadBufferSize(m_readBufferSize);
                 m_pSocket->setPauseMode(m_pauseMode);
 
-                makeConnections(m_pSocket.data());
+                makeConnections(m_pSocket);
                 setSocketState(QAbstractSocket::ConnectingState);
     #ifndef QT_NO_NETWORKPROXY
                 m_pSocket->setProxy(m_configuration.m_proxy);
@@ -440,8 +450,13 @@ void QWebSocketPrivate::ping(const QByteArray &payload)
 {
     QByteArray payloadTruncated = payload.left(125);
     m_pingTimer.restart();
+    quint32 maskingKey = 0;
+    if (m_mustMask)
+        maskingKey = generateMaskingKey();
     QByteArray pingFrame = getFrameHeader(QWebSocketProtocol::OpCodePing, payloadTruncated.size(),
-                                          0 /*do not mask*/, true);
+                                          maskingKey, true);
+    if (m_mustMask)
+        QWebSocketProtocol::mask(&payloadTruncated, maskingKey);
     pingFrame.append(payloadTruncated);
     qint64 ret = writeFrame(pingFrame);
     Q_UNUSED(ret);
@@ -537,6 +552,10 @@ void QWebSocketPrivate::makeConnections(const QTcpSocket *pTcpSocket)
         QObject::connect(pTcpSocket, &QAbstractSocket::aboutToClose, q, &QWebSocket::aboutToClose);
         QObject::connect(pTcpSocket, &QAbstractSocket::bytesWritten, q, &QWebSocket::bytesWritten);
 
+
+        QObjectPrivate::connect(pTcpSocket, &QObject::destroyed,
+                                this, &QWebSocketPrivate::socketDestroyed);
+
         //catch signals
         QObjectPrivate::connect(pTcpSocket, &QAbstractSocket::stateChanged, this,
                                 &QWebSocketPrivate::processStateChanged);
@@ -898,46 +917,66 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
     Q_Q(QWebSocket);
     if (Q_UNLIKELY(!pSocket))
         return;
+    // Reset handshake on a new connection.
+    if (m_handshakeState == AllDoneState)
+        m_handshakeState = NothingDoneState;
 
-    bool ok = false;
     QString errorDescription;
 
-    const QByteArray statusLine = pSocket->readLine();
-    int httpMajorVersion, httpMinorVersion;
-    int httpStatusCode;
-    QString httpStatusMessage;
-    if (Q_UNLIKELY(!parseStatusLine(statusLine, &httpMajorVersion, &httpMinorVersion,
-                                    &httpStatusCode, &httpStatusMessage))) {
-        errorDescription = QWebSocket::tr("Invalid statusline in response: %1.").arg(QString::fromLatin1(statusLine));
-    } else {
-        QString headerLine = readLine(pSocket);
-        QMap<QString, QString> headers;
-        while (!headerLine.isEmpty()) {
+    switch (m_handshakeState) {
+    case NothingDoneState:
+        m_headers.clear();
+        m_handshakeState = ReadingStatusState;
+        // no break
+    case ReadingStatusState:
+        if (!pSocket->canReadLine())
+            return;
+        m_statusLine = pSocket->readLine();
+        if (Q_UNLIKELY(!parseStatusLine(m_statusLine, &m_httpMajorVersion, &m_httpMinorVersion, &m_httpStatusCode, &m_httpStatusMessage))) {
+            errorDescription = QWebSocket::tr("Invalid statusline in response: %1.").arg(QString::fromLatin1(m_statusLine));
+            break;
+        }
+        m_handshakeState = ReadingHeaderState;
+        // no break
+    case ReadingHeaderState:
+        while (pSocket->canReadLine()) {
+            QString headerLine = readLine(pSocket);
             const QStringList headerField = headerLine.split(QStringLiteral(": "),
                                                              QString::SkipEmptyParts);
             if (headerField.size() == 2) {
-                headers.insertMulti(headerField[0].toLower(), headerField[1]);
+                m_headers.insertMulti(headerField[0].toLower(), headerField[1]);
+            }
+            if (headerField.isEmpty()) {
+                m_handshakeState = ParsingHeaderState;
+                break;
             }
-            headerLine = readLine(pSocket);
         }
 
-        const QString acceptKey = headers.value(QStringLiteral("sec-websocket-accept"),
-                                                QString());
-        const QString upgrade = headers.value(QStringLiteral("upgrade"), QString());
-        const QString connection = headers.value(QStringLiteral("connection"), QString());
+        if (m_handshakeState != ParsingHeaderState) {
+            if (pSocket->atEnd()) {
+                errorDescription = QWebSocket::tr("QWebSocketPrivate::processHandshake: Connection closed while reading header.");
+                break;
+            }
+            return;
+        }
+        // no break
+    case ParsingHeaderState: {
+        const QString acceptKey = m_headers.value(QStringLiteral("sec-websocket-accept"), QString());
+        const QString upgrade = m_headers.value(QStringLiteral("upgrade"), QString());
+        const QString connection = m_headers.value(QStringLiteral("connection"), QString());
 //        unused for the moment
-//        const QString extensions = headers.value(QStringLiteral("sec-websocket-extensions"),
+//        const QString extensions = m_headers.value(QStringLiteral("sec-websocket-extensions"),
 //                                                 QString());
-//        const QString protocol = headers.value(QStringLiteral("sec-websocket-protocol"),
+//        const QString protocol = m_headers.value(QStringLiteral("sec-websocket-protocol"),
 //                                               QString());
-        const QString version = headers.value(QStringLiteral("sec-websocket-version"),
-                                              QString());
+        const QString version = m_headers.value(QStringLiteral("sec-websocket-version"), QString());
 
-        if (Q_LIKELY(httpStatusCode == 101)) {
+        bool ok = false;
+        if (Q_LIKELY(m_httpStatusCode == 101)) {
             //HTTP/x.y 101 Switching Protocols
             //TODO: do not check the httpStatusText right now
             ok = !(acceptKey.isEmpty() ||
-                   (httpMajorVersion < 1 || httpMinorVersion < 1) ||
+                   (m_httpMajorVersion < 1 || m_httpMinorVersion < 1) ||
                    (upgrade.toLower() != QStringLiteral("websocket")) ||
                    (connection.toLower() != QStringLiteral("upgrade")));
             if (ok) {
@@ -950,9 +989,9 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
             } else {
                 errorDescription =
                     QWebSocket::tr("QWebSocketPrivate::processHandshake: Invalid statusline in response: %1.")
-                        .arg(QString::fromLatin1(statusLine));
+                        .arg(QString::fromLatin1(m_statusLine));
             }
-        } else if (httpStatusCode == 400) {
+        } else if (m_httpStatusCode == 400) {
             //HTTP/1.1 400 Bad Request
             if (!version.isEmpty()) {
                 const QStringList versions = version.split(QStringLiteral(", "),
@@ -963,29 +1002,38 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
                     errorDescription =
                             QWebSocket::tr("Handshake: Server requests a version that we don't support: %1.")
                             .arg(versions.join(QStringLiteral(", ")));
-                    ok = false;
                 } else {
                     //we tried v13, but something different went wrong
                     errorDescription =
                         QWebSocket::tr("QWebSocketPrivate::processHandshake: Unknown error condition encountered. Aborting connection.");
-                    ok = false;
                 }
+            } else {
+                    errorDescription =
+                        QWebSocket::tr("QWebSocketPrivate::processHandshake: Unknown error condition encountered. Aborting connection.");
             }
         } else {
             errorDescription =
                     QWebSocket::tr("QWebSocketPrivate::processHandshake: Unhandled http status code: %1 (%2).")
-                        .arg(httpStatusCode).arg(httpStatusMessage);
-            ok = false;
+                        .arg(m_httpStatusCode).arg(m_httpStatusMessage);
         }
+        if (ok)
+            m_handshakeState = AllDoneState;
+        break;
+    }
+    case AllDoneState:
+        Q_UNREACHABLE();
+        break;
+    }
 
-        if (!ok) {
-            setErrorString(errorDescription);
-            Q_EMIT q->error(QAbstractSocket::ConnectionRefusedError);
-        } else {
-            //handshake succeeded
-            setSocketState(QAbstractSocket::ConnectedState);
-            Q_EMIT q->connected();
-        }
+    if (m_handshakeState == AllDoneState) {
+        // handshake succeeded
+        setSocketState(QAbstractSocket::ConnectedState);
+        Q_EMIT q->connected();
+    } else {
+        // handshake failed
+        m_handshakeState = AllDoneState;
+        setErrorString(errorDescription);
+        Q_EMIT q->error(QAbstractSocket::ConnectionRefusedError);
     }
 }
 
@@ -997,6 +1045,10 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
     Q_ASSERT(m_pSocket);
     Q_Q(QWebSocket);
     QAbstractSocket::SocketState webSocketState = this->state();
+    int port = 80;
+    if (m_requestUrl.scheme() == QStringLiteral("wss"))
+        port = 443;
+
     switch (socketState) {
     case QAbstractSocket::ConnectedState:
         if (webSocketState == QAbstractSocket::ConnectingState) {
@@ -1005,7 +1057,7 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
                     createHandShakeRequest(m_resourceName,
                                            m_requestUrl.host()
                                                 % QStringLiteral(":")
-                                                % QString::number(m_requestUrl.port(80)),
+                                                % QString::number(m_requestUrl.port(port)),
                                            origin(),
                                            QString(),
                                            QString(),
@@ -1043,6 +1095,13 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
     }
 }
 
+void QWebSocketPrivate::socketDestroyed(QObject *socket)
+{
+    Q_ASSERT(m_pSocket);
+    if (m_pSocket == socket)
+        m_pSocket = Q_NULLPTR;
+}
+
 /*!
  \internal
  */
@@ -1051,9 +1110,9 @@ void QWebSocketPrivate::processData()
     Q_ASSERT(m_pSocket);
     while (m_pSocket->bytesAvailable()) {
         if (state() == QAbstractSocket::ConnectingState)
-            processHandshake(m_pSocket.data());
+            processHandshake(m_pSocket);
         else
-            m_dataProcessor.process(m_pSocket.data());
+            m_dataProcessor.process(m_pSocket);
     }
 }
 
